package com.jz.dmp.modules.controller.DataIntegration;

import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.utils.realTime.CmdUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceNameListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeTableInfoReq;
import com.jz.dmp.modules.model.DmpRealtimeSyncInfo;
import com.jz.dmp.modules.model.RealTimeSyncModel;
import com.jz.dmp.modules.service.DmpRealtimeSyncInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 实时同步任务
 *
 * @author Bellamy
 * @since 2021-01-05 10:56:18
 */
@RestController
@RequestMapping("/realTimeSync")
@Api(tags = "数据集成--实时同步任务")
public class RealTimeSyncController {

    private static Logger logger = LoggerFactory.getLogger(RealTimeSyncController.class);
    /**
     * 服务对象
     */
    @Autowired
    private DmpRealtimeSyncInfoService dmpRealtimeSyncInfoService;

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "实时同步任务列表分页查询", notes = "实时同步任务列表分页查询")
    @PostMapping(value = "/realTimeSyncListPage")
    public PageInfoResponse<RealTimeSyncListDto> getDataSourceListPage(@RequestBody @Validated RealTimeSyncListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<RealTimeSyncListDto> pageInfo = new PageInfoResponse<RealTimeSyncListDto>();
        if (StringUtils.isEmpty(req.getProjectId())) {
            pageInfo.setMessage("项目id不能为空！");
            pageInfo.setCode(ResultCode.PARAMS_ERROR);
            return pageInfo;
        }
        try {
            pageInfo = dmpRealtimeSyncInfoService.queryRealTimeSyncListPage(req);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 批量启动实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "批量启动运行实时同步任务", notes = "批量启动实时同步任务")
    @GetMapping(value = "/startRealTimeSync")
    @ApiImplicitParam(name = "realTaskId", value = "任务id")
    public JsonResult startRealTimeSync(@RequestParam String realTaskId) throws Exception {
        if (StringUtils.isEmpty(realTaskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        String[] ids = realTaskId.split(",");
        List<DmpRealtimeSyncInfo> list = dmpRealtimeSyncInfoService.queryListById(ids);
        if (list.size() > 0 && list != null) {
            for (int i = 0; i < list.size(); i++) {
                DmpRealtimeSyncInfo dmpRealtimeSyncInfo = list.get(i);
                String srcTopicName = dmpRealtimeSyncInfo.getSrcTopicName();
                System.out.println(srcTopicName);
                logger.info("############正常执行表数据id........" + ids[i]);
                String shellPath = "/app/bigdata-app/scripts/trigger_straming.sh";
                CmdUtils.callShell(shellPath, srcTopicName);
            }
        }
        return new JsonResult();
    }

    /**
     * 删除实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "删除实时同步任务", notes = "删除实时同步任务")
    @GetMapping(value = "/delRealTimeSync")
    @ApiImplicitParam(name = "realTaskId", value = "任务id")
    public JsonResult delRealTimeSync(@RequestParam String realTaskId) throws Exception {
        if (StringUtils.isEmpty(realTaskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        boolean jsonResult = dmpRealtimeSyncInfoService.deleteById(Integer.valueOf(realTaskId));
        if (jsonResult) {
            return new JsonResult();
        } else {
            return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR, "删除失败！");
        }
    }

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @ApiOperation(value = "获取数据源和目标数据源下拉框", notes = "获取数据源和目标数据源下拉框")
    @GetMapping(value = "/getDatasourceNameList")
    @ApiImplicitParams({@ApiImplicitParam(name = "projectId", value = "项目id",required = true),
            @ApiImplicitParam(name = "type", value = "数据源类型：01来源，02 目标源",required = true)})
    public JsonResult<List<DataSourceNameListDto>> getDatasourceNameList(@RequestParam String projectId, @RequestParam String type) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(type)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "类型不能为空！");
        }
        JsonResult<List<DataSourceNameListDto>> jsonResult = dmpRealtimeSyncInfoService.queryDatasourceNameList(projectId, type);
        return jsonResult;
    }

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @ApiOperation(value = "新增--选择源表信息", notes = "新增--选择源表信息")
    @PostMapping(value = "/getSourceDbTableList")
    public JsonResult<RealTimeSyncModel> getTableInfo(@RequestBody @Validated RealTimeTableInfoReq req, HttpServletRequest httpRequest) throws Exception {
        if (StringUtils.isEmpty(req.getProjectId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(req.getSrcDatasourceId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源数据源id不能为空！");
        }
        JsonResult<RealTimeSyncModel> jsonResult = dmpRealtimeSyncInfoService.queryTableInfoByParams(req);
        return jsonResult;
    }

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @ApiOperation(value = "新增--选择目标信息", notes = "新增--选择目标信息")
    @GetMapping(value = "/targetDatasourceInfo")
    @ApiImplicitParam(name = "projectId", value = "项目id",required = true)
    public JsonResult getTargetDatasourceInfo(@RequestParam String projectId) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        JsonResult jsonResult = dmpRealtimeSyncInfoService.selectTargetDatasourceInfo(projectId);
        return jsonResult;
    }

}